<?php
/**
 * Multi order for WooCommerce - Core Class
 *
 * @version 1.0.8
 * @since   1.0.0
 * @author  Algoritmika Ltd.
 */

if ( ! class_exists( 'Alg_MOWC_Core' ) ) {

	class Alg_MOWC_Core extends Alg_MOWC_WP_Plugin {

		/**
		 * Ensures only one instance is loaded or can be loaded.
		 *
		 * @version 1.0.1
		 * @since   1.0.1
		 * @return Alg_MOWC_Core
		 */
		public static function get_instance() {
			return parent::get_instance(); // TODO: Change the autogenerated stub
		}

		/**
		 * @var Alg_MOWC_Order_Bkg_Process
		 */
		public $order_bkg_process;

		/**
		 * @var Alg_MOWC_Order_Pay_Status_Bkg_Process;
		 */
		public $order_pay_status_bkg_process;

		/**
		 * Initializes the plugin.
		 *
		 * Should be called after the set_args() method
		 *
		 * @version 1.0.1
		 * @since   1.0.0
		 *
		 * @param array $args
		 */
		public function init() {
			parent::init();

			// Initializes order background process
			$this->order_bkg_process = new Alg_MOWC_Order_Bkg_Process();

			// Initializes order payment status background process
			$this->order_pay_status_bkg_process = new Alg_MOWC_Order_Pay_Status_Bkg_Process();

			// Init admin part
			if ( is_admin() ) {
				$this->init_admin_settings();
			}

			if ( filter_var( get_option( Alg_MOWC_Settings_General::OPTION_ENABLE_PLUGIN ), FILTER_VALIDATE_BOOLEAN ) ) {
				$this->setup_plugin();
			}
		}

		/**
		 * Initializes admin settings
		 *
		 * @version 1.0.0
		 * @since   1.0.0
		 */
		public function init_admin_settings() {
			new Alg_MOWC_Admin_Settings();
		}

		/**
		 * Setups the plugin
		 *
		 * @version 1.0.5
		 * @since   1.0.0
		 */
		public function setup_plugin() {
			// Payment status taxonomy
			/*$payment_status = new Alg_MOWC_Order_Payment_Status();
			$payment_status->create_taxonomy();
			$payment_status->setup_menu();
			*/

			// Payment status CMB
			/*$status_cmb = new Alg_MOWC_Payment_Status_CMB();
			$status_cmb->init();*/

			new Alg_MOWC_Multiorder_CMB();
			new Alg_MOWC_Order_Manager();
			new Alg_MOWC_Order_Columns();
			new Alg_MOWC_Orders_View();
			new Alg_MOWC_Orders_Search();
			new Alg_MOWC_Order_Item();
			new Alg_MOWC_Order_Actions();
			new Alg_MOWC_WC_Report();
		}

		/**
		 * Called when plugin is enabled
		 *
		 * @version 1.0.0
		 * @since   1.0.0
		 *
		 */
		public static function on_plugin_activation() {
			parent::on_plugin_activation();
			Alg_MOWC_Order_Manager::set_sort_order_meta();
			/*$payment_status = new Alg_MOWC_Order_Payment_Status();
			$payment_status->set_args();
			$payment_status->register();
			$payment_status->create_initial_status();*/
		}

	}
}